/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport.amqp;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.QueueingConsumer;
import com.rabbitmq.client.ShutdownSignalException;
import com.vii.brillien.ignition.BrillienContext;
import com.vii.brillien.ignition.transport.TransportStatics;
import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.transport.CommunicationListener;
import com.vii.brillien.kernel.axiom.transport.MessageProcessor;
import com.vii.streamline.services.ThreadServices;
import com.vii.streamline.services.json.JsonServices;

import java.util.logging.Level;

/**
 * AMQP Queue Listener implementation associated to an AmqpMediator instance.
 */
public class AmqpListener implements Runnable {

    /**
     * AmqpMediator instance
     */
    private AmqpMediator    mediator;

    /**
     * Name of the queue to be listened.
     */
    private String          queuename;

    /**
     * Flag flield to set the listening status of the Listener
     */
    private Boolean         listen;

    public AmqpListener(AmqpMediator mediator, String queuename) {
        this.mediator = mediator;
        this.queuename = queuename;
        listen = true;
    }

    /**
     * Get method for field listen
     */
    public boolean isListen() {
        return listen;
    }

    /**
     * Set method for field listen
     */
    public void setListen(boolean listen) {
        this.listen = listen;
    }

    @Override
    public void run() {
        BrillienContext.basicLog(Level.FINE, "Start listening on :: " + queuename );

        try{
            Channel ch = mediator.getConnection().createChannel();

            boolean autoAck = false;

            QueueingConsumer consumer = new QueueingConsumer( ch );

            ch.basicConsume(queuename, autoAck, consumer);

            while ( listen ) {
                QueueingConsumer.Delivery delivery;
                try {
                    delivery = consumer.nextDelivery(  );

                    BrillienContext.basicLog(Level.FINE, mediator.mediatorEntity() + ":: " + " at " + System.currentTimeMillis() + " Packet received: " + new String(delivery.getBody()), delivery.getProperties());

                    if( delivery.getProperties().getContentType() != null && delivery.getProperties().getContentType().equals( TransportStatics.TRANSPORT_MIME_TYPE ) ){
                        for( CommunicationListener<QueueingConsumer.Delivery> listener : mediator.getCommunicationListeners() ){
                            try{
                                listener.received( delivery );
                            }  catch(BrillienException e){
                                BrillienContext.exceptionLog( this.getClass().getName(), "run", e );
                            }
                        }

                        AmqpCommunication message = JsonServices.parseJSON( new String(delivery.getBody()), AmqpCommunication.class );

                        for( MessageProcessor processor : mediator.getNewMessageProcessorInstances( message ) ){
                            ThreadServices.getStpe().execute( processor );
                        }
                    }

                    ch.basicAck(delivery.getEnvelope().getDeliveryTag(), false);
                } catch (InterruptedException ie) {
                    continue;
                } catch (ShutdownSignalException se){
                    listen = false;
                    continue;
                }
            }
        } catch( Exception e ){
            BrillienContext.exceptionLog( this.getClass().getName(), "run", e );
        }
    }

}
